#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include "linux_endian.h"
#include <linux/gfs_ondisk.h>

#define pv(struct, member, fmt) printf("  "#member" = "fmt"\n", struct->member);
#define pa(struct, member, count) print_array(#member, struct->member, count);
static void
print_array(char *title, char *buf, int count)
{
        int x;

        printf("  %s =\n", title);
        for (x = 0; x < count; x++) {
                printf("%.2X ", (unsigned char) buf[x]);
                if (x % 16 == 15)
                        printf("\n");
        }
        if (x % 16)
                printf("\n");

}


/*
#define CPIN_08(s1, s2, member, count) {memcpy((s1->member), (s2->member), (count));}
#define CPOUT_08(s1, s2, member, count) {memcpy((s2->member), (s1->member), (count));}
#define CPIN_16(s1, s2, member) {(s1->member) = gfs16_to_cpu((s2->member));}
#define CPOUT_16(s1, s2, member) {(s2->member) = cpu_to_gfs16((s1->member));}
#define CPIN_32(s1, s2, member) {(s1->member) = gfs32_to_cpu((s2->member));}
#define CPOUT_32(s1, s2, member) {(s2->member) = cpu_to_gfs32((s1->member));}
#define CPIN_64(s1, s2, member) {(s1->member) = gfs64_to_cpu((s2->member));}
#define CPOUT_64(s1, s2, member) {(s2->member) = cpu_to_gfs64((s1->member));}

void
gfs_inum_in(struct gfs_inum *no, char *buf)
{
        struct gfs_inum *str = (struct gfs_inum *) buf;

        CPIN_64(no, str, no_formal_ino);
        CPIN_64(no, str, no_addr);

}

void
gfs_meta_header_in(struct gfs_meta_header *mh, char *buf)
{
        struct gfs_meta_header *str = (struct gfs_meta_header *) buf;

        CPIN_32(mh, str, mh_magic);
        CPIN_32(mh, str, mh_type);
        CPIN_64(mh, str, mh_generation);
        CPIN_32(mh, str, mh_format);
        CPIN_32(mh, str, mh_incarn);

}

void
gfs_dinode_in(struct gfs_dinode *dinode, char *buf)
{
        struct gfs_dinode *str = (struct gfs_dinode *) buf;

        gfs_meta_header_in(&dinode->di_header, buf);

        gfs_inum_in(&dinode->di_num, (char *) &str->di_num);

        CPIN_32(dinode, str, di_mode);
        CPIN_32(dinode, str, di_uid);
        CPIN_32(dinode, str, di_gid);
        CPIN_32(dinode, str, di_nlink);
        CPIN_64(dinode, str, di_size);
        CPIN_64(dinode, str, di_blocks);
        CPIN_64(dinode, str, di_atime);
        CPIN_64(dinode, str, di_mtime);
        CPIN_64(dinode, str, di_ctime);
        CPIN_32(dinode, str, di_major);
        CPIN_32(dinode, str, di_minor);

        CPIN_64(dinode, str, di_rgrp);
        CPIN_64(dinode, str, di_goal_rgrp);
        CPIN_32(dinode, str, di_goal_dblk);
        CPIN_32(dinode, str, di_goal_mblk);
        CPIN_32(dinode, str, di_flags);
        CPIN_32(dinode, str, di_payload_format);
        CPIN_16(dinode, str, di_type);
        CPIN_16(dinode, str, di_height);
        CPIN_32(dinode, str, di_incarn);
        CPIN_16(dinode, str, di_pad);

        CPIN_16(dinode, str, di_depth);
        CPIN_32(dinode, str, di_entries);

        gfs_inum_in(&dinode->di_next_unused, (char *) &str->di_next_unused);

        CPIN_64(dinode, str, di_eattr);

        CPIN_08(dinode, str, di_reserved, 56);

}
*/
void
gfs_inum_print(struct gfs_inum *no)
{

        pv(no, no_formal_ino, "%" PRIu64);
        pv(no, no_addr, "%" PRIu64);

}

void
gfs_meta_header_print(struct gfs_meta_header *mh)
{

        pv(mh, mh_magic, "0x%.8X");
        pv(mh, mh_type, "%u");
        pv(mh, mh_generation, "%" PRIu64);
        pv(mh, mh_format, "%u");
        pv(mh, mh_incarn, "%u");

}

void
gfs_dinode_print(struct gfs_dinode *di)
{

        gfs_meta_header_print(&di->di_header);

        gfs_inum_print(&di->di_num);

        pv(di, di_mode, "0%o");
        pv(di, di_uid, "%u");
        pv(di, di_gid, "%u");
        pv(di, di_nlink, "%u");
        pv(di, di_size, "%" PRIu64);
        pv(di, di_blocks, "%" PRIu64);
        pv(di, di_atime, "%" PRId64);
        pv(di, di_mtime, "%" PRId64);
        pv(di, di_ctime, "%" PRId64);
        pv(di, di_major, "%u");
        pv(di, di_minor, "%u");

        pv(di, di_rgrp, "%" PRIu64);
        pv(di, di_goal_rgrp, "%" PRIu64);
        pv(di, di_goal_dblk, "%u");
        pv(di, di_goal_mblk, "%u");
        pv(di, di_flags, "0x%.8X");
        pv(di, di_payload_format, "%u");
        pv(di, di_type, "%u");
        pv(di, di_height, "%u");
        pv(di, di_incarn, "%u");
        pv(di, di_pad, "%u");

        pv(di, di_depth, "%u");
        pv(di, di_entries, "%u");

        gfs_inum_print(&di->di_next_unused);

	pv(di, di_eattr, "%" PRIu64);

        pa(di, di_reserved, 56);

}
